define(['dojo/_base/declare', 'jimu/BaseWidget', 'dojo/on', 'dojo/dom', 'dojo/_base/array', 
        'esri/geometry/Point', 'esri/layers/FeatureLayer', 'esri/graphic', 'esri/SpatialReference',
        'esri/Color','esri/symbols/SimpleMarkerSymbol'],
function(declare, BaseWidget, on, dom, array, Point, FeatureLayer, graphic, SpatialReference, 
         Color, SimpleMarkerSymbol) {
	return declare([BaseWidget], {
	    myLayer: null,  //  このウィジェットで扱うLayer
	    myLayerSymbol: new SimpleMarkerSymbol().setStyle(
            SimpleMarkerSymbol.STYLE_SQUARE).setColor(
            new Color([255,0,0,0.5])),
        startup: function() {
            var button = dom.byId('btnCsvLoaderRead'),
                file = dom.byId('inputCsvLoaderFile');
            var pointDefinition = {
                "geometryType": "esriGeometryPoint",
                "fields": [{
                    "name": "CSV読込",
                    "type": "esriFieldTypeInteger"
                }]
            };
            this.myLayer = new FeatureLayer({
                layerDefinition: pointDefinition,
                featureSet: null
            });
            //this.myLayer = new GraphicsLayer();
            this.map.addLayer(this.myLayer);
            
            this.own(
                on(button, 'click', dojo.hitch(this, function(evt) {
                    var reader = new FileReader();
                    var widgetObj = this;
                    reader.onload = function(){
                        var text = reader.result;
                        var dataArray = widgetObj.parseCsv(text);
                        var dataArrayLength = dataArray.length;
                        widgetObj.myLayer.clear();
                        for (var i = 0; i < dataArrayLength; i++)
                            widgetObj.addObjToLayer(dataArray[i]);
                    };
                    reader.readAsText(file.files[0]);
                }))
            );
        },
        parseCsv: function(text) {
            // textをCSV(１行目は見出し行)とみなして分割
            // →オブジェクト配列に詰めて返す
            var lines = text.replace(/\r\n/g,'\n').split('\n');
            if (lines.length < 1) return null;
            
            // まず列名を取得
            var headerColumns= lines[0].split(',');
            // 緯度、経度に相当する列の位置を探す(複数該当したら最初の列名を利用)
            var latColumnIndex = -1,
                lonColumnIndex = -1;
            for (var i = 0; i < headerColumns.length; i++) {
                if (latColumnIndex === -1 && array.indexOf(['Y', '緯度', 'lat', 'latitude'], headerColumns[i]) >= 0)
                    latColumnIndex = i;
                if (lonColumnIndex === -1 && array.indexOf(['X', '経度', 'lon', 'longitude'], headerColumns[i]) >= 0)
                    lonColumnIndex = i;
            }
            if (latColumnIndex < 0 || lonColumnIndex < 0) {
                alert('CSVファイルには、緯度列と経度列が必要です。');
                return null;
            }
            
            // データをオブジェクト配列に詰めなおして返す(各行の列数が一定と仮定)
            var objArray = [];
            var lineLength = lines.length;
            var headerColumnsLength = headerColumns.length
            for (var i = 1; i < lineLength; i++) {
                var cells = lines[i].split(',');
                var obj = {_x: cells[lonColumnIndex], _y: cells[latColumnIndex]}; // 呼び出し元で使うので位置情報は固定の名前で持っておく
                for (var j = 0; j < headerColumnsLength; j++) {
                    obj[headerColumns[j]] = cells[j];
                }
                objArray.push(obj);
            }
            return objArray;
        },
        addObjToLayer: function(obj) {
            // 緯度、経度をそれぞれ_y,_xプロパティに持っているオブジェクトを受け取り
            // Graphicオブジェクトに変換したうえで自前のGraphicLayerに追加する
            var geometry = new Point(obj["_x"], obj["_y"],  new SpatialReference({ wkid: 4326 }));
            var g = new graphic(geometry, this.myLayerSymbol, obj);
            this.myLayer.add(g);
        }
	});
});